public class UE11Aufgabe16 {

	public static void main(String[] args) {
		OpalaSpieler weiss = new OpalaSpieler(OpalaFigur.WEISS);
		OpalaSpieler schwarz = new OpalaSpieler(OpalaFigur.SCHWARZ);
		OpalaBrett brett = new OpalaBrett();
		OpalaRegeln regeln = new OpalaRegeln(brett, weiss, schwarz);

		OpalaSpiel opala = new OpalaSpiel(weiss, schwarz, brett, regeln);
		opala.spielen();
	}

}

// -----------------------------------------------------
// Klasse OpalaSpiel 
// -----------------------------------------------------
class OpalaSpiel {

	// Attribute
	OpalaSpieler spielerWeiss, spielerSchwarz;
	OpalaBrett brett;
	OpalaRegeln regeln;

	// Methoden

	// Konstruktor (Initialisierung eines Spiels)
	OpalaSpiel(OpalaSpieler a, OpalaSpieler b, OpalaBrett brett,
			OpalaRegeln regeln) {
		this.spielerWeiss = a;
		this.spielerSchwarz = b;
		this.brett = brett;
		this.regeln = regeln;
	}

	// Durchfuehrung eines Spiels
	void spielen() {
		// Spieler Weiss beginnt
		OpalaSpieler aktSpieler = this.spielerWeiss;
		OpalaSpielzug zug = null;
		this.brett.gebeSpielbrettAus();

		// abwechselndes Ziehen bis Spielende erreicht ist
		int sieger = OpalaRegeln.KEIN_ENDE;
		do {
			if (aktSpieler == this.spielerWeiss) {
				IO.println("Spieler Weiss ist am Zug!");
			} else {
				IO.println("Spieler Schwarz ist am Zug!");
			}

			// legalen Spielzug ermitteln und ausfuehren
			zug = aktSpieler.liefereNaechstenSpielzug(zug);
			while (!this.regeln.spielzugOK(zug, aktSpieler)) {
				IO.println("Ungueltig; neue Eingabe!");
				zug = aktSpieler.liefereNaechstenSpielzug(zug);
			}
			this.brett.fuehreSpielzugAus(zug);
			this.brett.gebeSpielbrettAus();

			if ((sieger = this.regeln.spielEnde(aktSpieler, zug)) > OpalaRegeln.KEIN_ENDE) {
				break;
			}

			// der andere Spieler ist nun am Zug
			if (aktSpieler == this.spielerWeiss) {
				aktSpieler = this.spielerSchwarz;
			} else {
				aktSpieler = this.spielerWeiss;
			}
		} while (true);

		// Spiel ist zuende
		if (sieger == OpalaRegeln.SIEGER_WEISS) {
			IO.println("Spielende! Sieger ist Spieler A (WEISS)");
		} else {
			IO.println("Spielende! Sieger ist Spieler B (SCHWARZ)");
		}
	}
}

// -----------------------------------------------------
// Klasse OpalaBrett
// -----------------------------------------------------
class OpalaBrett {

	/*
	 * gewaehlte Datenstruktur:
	 * 
	 * 0 1 2 3 4 5 +-+-+-+-+-+-+ 0 |S|T|C|C|T|S| 6 +-+-+-+-+-+-+ 1 |s|t|c|c|t|s|
	 * 5 +-+-+-+-+-+-+ 2 | | | | | | | 4 +-+-+-+-+-+-+ 3 | | | | | | | 3
	 * +-+-+-+-+-+-+ 4 |q|d|k|k|d|q| 2 +-+-+-+-+-+-+ 5 |Q|D|K|K|D|Q| 1
	 * +-+-+-+-+-+-+ A B C D E F
	 * 
	 * repraesentiert als 8 x 8 - Matrix; Schwarze Spielfiguren: S = grosses
	 * Quadrat (Square) s = kleines Quadrat (square) T = grosses Dreieck
	 * (Triangle) t = kleines Dreieck (triangle) C = grosser Kreis (Circle) c =
	 * kleiner Kreis (circle)
	 * 
	 * Weisse Spielfiguren: Q = grosses Quadrat q = kleines Quadrat D = grosses
	 * Dreieck d = kleines Dreieck K = grosser Kreis k = kleiner Kreis
	 */

	// Konstanten
	final static int GROESSE = 6;

	// Attribute
	OpalaFeld[][] brett;

	// Methoden
	// Konstruktor (Initialisierung des Opala-Brettes)
	OpalaBrett() {
		this.brett = new OpalaFeld[OpalaBrett.GROESSE][OpalaBrett.GROESSE];
		for (int z = 0; z < OpalaBrett.GROESSE; z++) {
			for (int s = 0; s < OpalaBrett.GROESSE; s++) {
				this.brett[z][s] = new OpalaFeld(z, s);
			}
		}

		// mit schwarzen Figuren besetzte Felder
		this.brett[0][0] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.QUADRAT_GROSS_SCHWARZ), 0, 0);

		this.brett[0][1] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.DREIECK_GROSS_SCHWARZ), 0, 1);

		this.brett[0][2] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.KREIS_GROSS_SCHWARZ), 0, 2);

		this.brett[0][3] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.KREIS_GROSS_SCHWARZ), 0, 3);

		this.brett[0][4] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.DREIECK_GROSS_SCHWARZ), 0, 4);

		this.brett[0][5] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.QUADRAT_GROSS_SCHWARZ), 0, 5);

		this.brett[1][0] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.QUADRAT_KLEIN_SCHWARZ), 1, 0);

		this.brett[1][1] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.DREIECK_KLEIN_SCHWARZ), 1, 1);

		this.brett[1][2] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.KREIS_KLEIN_SCHWARZ), 1, 2);

		this.brett[1][3] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.KREIS_KLEIN_SCHWARZ), 1, 3);

		this.brett[1][4] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.DREIECK_KLEIN_SCHWARZ), 1, 4);

		this.brett[1][5] = new OpalaFeld(new OpalaFigur(OpalaFigur.SCHWARZ,
				OpalaFigur.QUADRAT_KLEIN_SCHWARZ), 1, 5);

		// mit weissen Figuren besetzte Felder
		this.brett[OpalaBrett.GROESSE - 1][0] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.QUADRAT_GROSS_WEISS),
				OpalaBrett.GROESSE - 1, 0);

		this.brett[OpalaBrett.GROESSE - 1][1] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.DREIECK_GROSS_WEISS),
				OpalaBrett.GROESSE - 1, 1);

		this.brett[OpalaBrett.GROESSE - 1][2] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.KREIS_GROSS_WEISS),
				OpalaBrett.GROESSE - 1, 2);

		this.brett[OpalaBrett.GROESSE - 1][3] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.KREIS_GROSS_WEISS),
				OpalaBrett.GROESSE - 1, 3);

		this.brett[OpalaBrett.GROESSE - 1][4] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.DREIECK_GROSS_WEISS),
				OpalaBrett.GROESSE - 1, 4);

		this.brett[OpalaBrett.GROESSE - 1][5] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.QUADRAT_GROSS_WEISS),
				OpalaBrett.GROESSE - 1, 5);

		this.brett[OpalaBrett.GROESSE - 2][0] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.QUADRAT_KLEIN_WEISS),
				OpalaBrett.GROESSE - 2, 0);

		this.brett[OpalaBrett.GROESSE - 2][1] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.DREIECK_KLEIN_WEISS),
				OpalaBrett.GROESSE - 2, 1);

		this.brett[OpalaBrett.GROESSE - 2][2] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.KREIS_KLEIN_WEISS),
				OpalaBrett.GROESSE - 2, 2);

		this.brett[OpalaBrett.GROESSE - 2][3] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.KREIS_KLEIN_WEISS),
				OpalaBrett.GROESSE - 2, 3);

		this.brett[OpalaBrett.GROESSE - 2][4] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.DREIECK_KLEIN_WEISS),
				OpalaBrett.GROESSE - 2, 4);

		this.brett[OpalaBrett.GROESSE - 2][5] = new OpalaFeld(new OpalaFigur(
				OpalaFigur.WEISS, OpalaFigur.QUADRAT_KLEIN_WEISS),
				OpalaBrett.GROESSE - 2, 5);

	}

	// Ausfuehrung eines Spielzugs auf dem Brett
	void fuehreSpielzugAus(OpalaSpielzug zug) {
		int vonZeile = zug.vonWo().liefereZeile();
		int vonSpalte = zug.vonWo().liefereSpalte();
		int nachZeile = zug.nachWohin().liefereZeile();
		int nachSpalte = zug.nachWohin().liefereSpalte();

		OpalaFigur vonFigur = this.brett[vonZeile][vonSpalte].liefereFigur();

		this.brett[vonZeile][vonSpalte].entferneFigur();

		if (!this.brett[nachZeile][nachSpalte].istLeer()) { // Schlagen
			this.brett[nachZeile][nachSpalte].entferneFigur();
		}

		this.brett[nachZeile][nachSpalte].setzeFigur(vonFigur);

	}

	// Copy-Konstruktor
	OpalaBrett(OpalaBrett exBrett) {
		this.brett = new OpalaFeld[OpalaBrett.GROESSE][OpalaBrett.GROESSE];
		for (int z = 0; z < OpalaBrett.GROESSE; z++) {
			for (int s = 0; s < OpalaBrett.GROESSE; s++) {
				this.brett[z][s] = new OpalaFeld(exBrett.brett[z][s]);
			}
		}
	}

	// Ausgabe des aktuellen Spielbretts
	void gebeSpielbrettAus() {

		IO.println();
		IO.println("  Spieler B (Schwarz)");
		IO.println();
		IO.print("  ");
		for (int j = 0; j < OpalaBrett.GROESSE; j++) {
			IO.print((char) ('A' + j));
			IO.print(" ");
		}
		IO.println();

		IO.print(" ");
		for (int j = 0; j < OpalaBrett.GROESSE; j++) {
			IO.print("--");
		}
		IO.println();

		for (int i = 0; i < OpalaBrett.GROESSE; i++) {
			IO.print(OpalaBrett.GROESSE - i);
			IO.print("|");
			for (int j = 0; j < OpalaBrett.GROESSE; j++) {
				IO.print(this.liefereZeichen(i, j));
				IO.print("|");
			}
			IO.println();

			IO.print(" ");
			for (int j = 0; j < OpalaBrett.GROESSE; j++) {
				IO.print("--");
			}
			IO.println();
		}
		IO.println();
		IO.println("  Spieler A (WEISS)");
		IO.println();

	}

	// Lieferung des Feldes an der Stelle pos
	OpalaFeld liefereFeld(Position pos) {
		return this.brett[pos.liefereZeile()][pos.liefereSpalte()];
	}

	// Lieferung des Feldes an der Stelle z/s
	OpalaFeld liefereFeld(int z, int s) {
		return this.brett[z][s];
	}

	// Lieferung des Figurzeichens oder Leerzeichen, falls
	// leeres Feld
	char liefereZeichen(int z, int s) {
		if (this.brett[z][s].istLeer()) {
			return ' ';
		} else {
			return this.brett[z][s].liefereFigur().liefereZeichen();
		}
	}

	// liefert alle Figuren eines Spielers
	OpalaFigur[] liefereFiguren(OpalaSpieler spieler) {
		OpalaFigur[] figuren = new OpalaFigur[OpalaFigur.ANZAHL_SPIELER_FIGUREN];
		int pos = 0;
		for (int z = 0; z < OpalaBrett.GROESSE; z++) {
			for (int s = 0; s < OpalaBrett.GROESSE; s++) {
				if (!this.brett[z][s].istLeer()
						&& this.brett[z][s].liefereFigur().istWeiss() == spieler
								.istWeiss()) {
					figuren[pos++] = this.brett[z][s].liefereFigur();
				}
			}
		}
		// umkopieren
		OpalaFigur[] figuren2 = new OpalaFigur[pos];
		for (int i = 0; i < pos; i++) {
			figuren2[i] = figuren[i];
		}
		return figuren2;
	}

	// liefert Anzahl an aktuellen Spielfiguren eines Spielers
	int anzahlFiguren(OpalaSpieler spieler) {
		int anzahl = 0;
		for (int z = 0; z < OpalaBrett.GROESSE; z++) {
			for (int s = 0; s < OpalaBrett.GROESSE; s++) {
				if (!this.brett[z][s].istLeer()
						&& this.brett[z][s].liefereFigur().istWeiss() == spieler
								.istWeiss()) {
					anzahl++;
				}
			}
		}
		return anzahl;
	}

}

// -----------------------------------------------------
// Klasse OpalaFeld
// -----------------------------------------------------
class OpalaFeld {

	int zeile, spalte;
	OpalaFigur figur;

	OpalaFeld(int zeile, int spalte) {
		// leeres Feld
		this.figur = null;
		this.zeile = zeile;
		this.spalte = spalte;
	}

	// Copy-Konstruktor
	OpalaFeld(OpalaFeld exFeld) {
		this.zeile = exFeld.zeile;
		this.spalte = exFeld.spalte;
		if (exFeld.figur == null) {
			this.figur = null;
		} else {
			this.figur = new OpalaFigur(exFeld.figur);
			this.figur.neuesFeld(this);
		}
	}

	OpalaFeld(OpalaFigur figur, int zeile, int spalte) {
		this.figur = figur;
		this.zeile = zeile;
		this.spalte = spalte;
		this.figur.neuesFeld(this);
	}

	OpalaFigur liefereFigur() {
		return this.figur;
	}

	int liefereZeile() {
		return this.zeile;
	}

	int liefereSpalte() {
		return this.spalte;
	}

	boolean istLeer() {
		return this.figur == null;
	}

	OpalaFigur entferneFigur() {
		this.figur.schlagen();
		OpalaFigur fig = this.figur;
		this.figur = null;
		return fig;
	}

	void setzeFigur(OpalaFigur figur) {
		this.figur = figur;
		this.figur.neuesFeld(this);
	}

}

// -----------------------------------------------------
// Klasse OpalaFigur
// -----------------------------------------------------
class OpalaFigur {

	// Konstanten
	final static boolean WEISS = true;
	final static boolean SCHWARZ = !OpalaFigur.WEISS;

	final static int ANZAHL_FIGUREN = 24;
	final static int ANZAHL_SPIELER_FIGUREN = OpalaFigur.ANZAHL_FIGUREN / 2;

	final static char QUADRAT_GROSS_SCHWARZ = 'S';
	final static char DREIECK_GROSS_SCHWARZ = 'T';
	final static char KREIS_GROSS_SCHWARZ = 'C';

	final static char QUADRAT_KLEIN_SCHWARZ = 's';
	final static char DREIECK_KLEIN_SCHWARZ = 't';
	final static char KREIS_KLEIN_SCHWARZ = 'c';

	final static char QUADRAT_GROSS_WEISS = 'Q';
	final static char DREIECK_GROSS_WEISS = 'D';
	final static char KREIS_GROSS_WEISS = 'K';

	final static char QUADRAT_KLEIN_WEISS = 'q';
	final static char DREIECK_KLEIN_WEISS = 'd';
	final static char KREIS_KLEIN_WEISS = 'k';

	// Attribute
	boolean istWeiss;
	char zeichen;
	OpalaFeld feld;

	// Methoden
	// Konstruktor (Initialisierung einer Figur)
	OpalaFigur(boolean istWeiss, char zeichen) {
		this.istWeiss = istWeiss;
		this.zeichen = zeichen;
		// anschliessend wird direkt neuesFeld aufgerufen und damit this.feld
		// initialisiert
	}

	// Copy-Konstruktor
	OpalaFigur(OpalaFigur exFigur) {
		this.istWeiss = exFigur.istWeiss;
		this.zeichen = exFigur.zeichen;
		// anschliessend wird direkt neuesFeld aufgerufen und damit this.feld
		// initialisiert
	}

	boolean istWeiss() {
		return this.istWeiss;
	}

	void schlagen() {
		this.feld = null;
	}

	boolean istGeschlagen() {
		return this.feld == null;
	}

	int liefereZeile() {
		if (this.feld != null) {
			return this.feld.liefereZeile();
		}
		return -1;
	}

	int liefereSpalte() {
		if (this.feld != null) {
			return this.feld.liefereSpalte();
		}
		return -1;
	}

	char liefereZeichen() {
		return this.zeichen;
	}

	OpalaFeld liefereFeld() {
		return this.feld;
	}

	void neuesFeld(OpalaFeld feld) {
		this.feld = feld;
	}

	boolean istQuadratGross() {
		return this.zeichen == OpalaFigur.QUADRAT_GROSS_SCHWARZ
				|| this.zeichen == OpalaFigur.QUADRAT_GROSS_WEISS;
	}

	boolean istDreieckGross() {
		return this.zeichen == OpalaFigur.DREIECK_GROSS_SCHWARZ
				|| this.zeichen == OpalaFigur.DREIECK_GROSS_WEISS;
	}

	boolean istKreisGross() {
		return this.zeichen == OpalaFigur.KREIS_GROSS_SCHWARZ
				|| this.zeichen == OpalaFigur.KREIS_GROSS_WEISS;
	}

	boolean istQuadratKlein() {
		return this.zeichen == OpalaFigur.QUADRAT_KLEIN_SCHWARZ
				|| this.zeichen == OpalaFigur.QUADRAT_KLEIN_WEISS;
	}

	boolean istDreieckKlein() {
		return this.zeichen == OpalaFigur.DREIECK_KLEIN_SCHWARZ
				|| this.zeichen == OpalaFigur.DREIECK_KLEIN_WEISS;
	}

	boolean istKreisKlein() {
		return this.zeichen == OpalaFigur.KREIS_KLEIN_SCHWARZ
				|| this.zeichen == OpalaFigur.KREIS_KLEIN_WEISS;
	}

}

// -----------------------------------------------------
// Klasse Position
// -----------------------------------------------------

class Position {

	// Attribute
	int zeile; // 0 - OpalaBrett.GROESSE-1
	int spalte; // 0 - OpalaBrett.GROESSE-1

	// Methoden
	// Konstruktor (Initialisierung gemaess interner Repraesentation)
	Position(int z, int s) {
		this.zeile = z;
		this.spalte = s;
	}

	// Konstruktor (Initialisierung gemaess externer Repraesentation)
	Position(char z, char s) {
		// z zwischen '6' (GROESSE) und '1'
		this.zeile = OpalaBrett.GROESSE - (z - '0');
		// s zwischen 'A' und 'F'
		this.spalte = s - 'A';
	}

	int liefereZeile() {
		return this.zeile;
	}

	int liefereSpalte() {
		return this.spalte;
	}

	boolean identisch(Position andere) {
		return this.zeile == andere.zeile && this.spalte == andere.spalte;
	}

}

// -----------------------------------------------------
// Klasse OpalaSpielzug
// -----------------------------------------------------

class OpalaSpielzug {

	// Attribute
	Position von;
	Position nach;

	// Methoden
	// Konstruktor (Initialisierung eines Spielzugs)
	OpalaSpielzug(Position von, Position nach) {
		this.von = von;
		this.nach = nach;
	}

	// Konstruktor (Initialisierung eines Spielzugs)
	OpalaSpielzug(int vonZeile, int vonSpalte, int nachZeile, int nachSpalte) {
		this.von = new Position(vonZeile, vonSpalte);
		this.nach = new Position(nachZeile, nachSpalte);
	}

	// Konstruktor (Initialisierung eines Spielzugs)
	OpalaSpielzug(char vonZeile, char vonSpalte, char nachZeile, char nachSpalte) {
		this.von = new Position(vonZeile, vonSpalte);
		this.nach = new Position(nachZeile, nachSpalte);
	}

	Position vonWo() {
		return this.von;
	}

	Position nachWohin() {
		return this.nach;
	}

}

// -----------------------------------------------------
// Klasse OpalaRegeln
// -----------------------------------------------------
class OpalaRegeln {

	// Attribute
	OpalaBrett brett;
	OpalaSpieler weiss;
	OpalaSpieler schwarz;
	OpalaFigur merkFigur;

	// Konstanten
	final static int KEIN_ENDE = 0;
	final static int SIEGER_WEISS = 1;
	final static int SIEGER_SCHWARZ = 2;

	// Methoden
	// Konstruktor (Initialisierung der Regeln)
	OpalaRegeln(OpalaBrett brett, OpalaSpieler weiss, OpalaSpieler schwarz) {
		this.brett = brett;
		this.weiss = weiss;
		this.schwarz = schwarz;
		this.merkFigur = null;
	}

	// Ueberpruefung eines Spielzugs
	boolean spielzugOK(OpalaSpielzug zug, OpalaSpieler spieler) {
		// korrekte Felder?
		int vonZeile, vonSpalte, nachZeile, nachSpalte;

		if ((vonZeile = zug.vonWo().liefereZeile()) < 0
				|| (vonSpalte = zug.vonWo().liefereSpalte()) < 0
				|| vonZeile >= OpalaBrett.GROESSE
				|| vonSpalte >= OpalaBrett.GROESSE
				|| (nachZeile = zug.nachWohin().liefereZeile()) < 0
				|| (nachSpalte = zug.nachWohin().liefereSpalte()) < 0
				|| nachZeile >= OpalaBrett.GROESSE
				|| nachSpalte >= OpalaBrett.GROESSE) {
			return false;
		}

		// Figur darf nicht stehenbleiben !
		if (zug.vonWo().identisch(zug.nachWohin())) {
			return false;
		}

		// auf dem von-Feld muss eine eigene Spielfigur stehen
		OpalaFeld vonFeld = this.brett.liefereFeld(zug.vonWo());
		OpalaFigur vonFigur = vonFeld.liefereFigur();
		if (vonFeld.istLeer() || spieler.istWeiss() != vonFigur.istWeiss()) {
			return false;
		}

		// auf dem nach-Feld darf keine eigene Spielfigur stehen
		OpalaFeld nachFeld = this.brett.liefereFeld(zug.nachWohin());
		OpalaFigur nachFigur = nachFeld.liefereFigur();
		if (!nachFeld.istLeer() && spieler.istWeiss() == nachFigur.istWeiss()) {
			return false;
		}

		// gueltiger Zug gemaess Zugregeln fuer Figuren
		if (vonFigur.istQuadratGross()) {
			if (!(vonZeile == nachZeile || vonSpalte == nachSpalte)) {
				return false;
			}
		} else if (vonFigur.istQuadratKlein()) {
			if (!(OpalaRegeln.abs(vonZeile - nachZeile)
					+ OpalaRegeln.abs(vonSpalte - nachSpalte) == 1)) {
				return false;
			}
		} else if (vonFigur.istDreieckGross()) {
			if (!(OpalaRegeln.abs(vonZeile - nachZeile) == OpalaRegeln
					.abs(vonSpalte - nachSpalte))) {
				return false;
			}
		} else if (vonFigur.istDreieckKlein()) {
			if (OpalaRegeln.abs(vonZeile - nachZeile) != 1
					|| OpalaRegeln.abs(vonSpalte - nachSpalte) != 1) {
				return false;
			}
		} else if (vonFigur.istKreisGross()) {
			if (!(vonZeile == nachZeile || vonSpalte == nachSpalte || OpalaRegeln
					.abs(vonZeile - nachZeile) == OpalaRegeln.abs(vonSpalte
					- nachSpalte))) {
				return false;
			}
		} else if (vonFigur.istKreisKlein()) {
			if (OpalaRegeln.abs(vonZeile - nachZeile) > 1
					|| OpalaRegeln.abs(vonSpalte - nachSpalte) > 1) {
				return false;
			}
		}

		// es duerfen keine Spielfiguren uebersprungen werden
		if (vonFigur.istQuadratGross()) {
			if (vonZeile < nachZeile) {
				for (int i = vonZeile + 1; i < nachZeile; i++) {
					if (!this.brett.liefereFeld(i, vonSpalte).istLeer()) {
						return false;
					}
				}
			} else if (vonZeile > nachZeile) {
				for (int i = vonZeile - 1; i > nachZeile; i--) {
					if (!this.brett.liefereFeld(i, vonSpalte).istLeer()) {
						return false;
					}
				}
			} else if (vonSpalte < nachSpalte) {
				for (int i = vonSpalte + 1; i < nachSpalte; i++) {
					if (!this.brett.liefereFeld(vonZeile, i).istLeer()) {
						return false;
					}
				}
			} else if (vonSpalte > nachSpalte) {
				for (int i = vonSpalte - 1; i > nachSpalte; i--) {
					if (!this.brett.liefereFeld(vonZeile, i).istLeer()) {
						return false;
					}
				}
			}
		} else if (vonFigur.istDreieckGross()) {
			if (vonZeile < nachZeile && vonSpalte < nachSpalte) {
				for (int i = 1; i < nachZeile - vonZeile; i++) {
					if (!this.brett.liefereFeld(vonZeile + i, vonSpalte + i)
							.istLeer()) {
						return false;
					}
				}
			} else if (vonZeile > nachZeile && vonSpalte < nachSpalte) {
				for (int i = 1; i < vonZeile - nachZeile; i++) {
					if (!this.brett.liefereFeld(vonZeile - i, vonSpalte + i)
							.istLeer()) {
						return false;
					}
				}
			} else if (vonZeile < nachZeile && vonSpalte > nachSpalte) {
				for (int i = 1; i < nachZeile - vonZeile; i++) {
					if (!this.brett.liefereFeld(vonZeile + i, vonSpalte - i)
							.istLeer()) {
						return false;
					}
				}
			} else { // beide >
				for (int i = 1; i < vonZeile - nachZeile; i++) {
					if (!this.brett.liefereFeld(vonZeile - i, vonSpalte - i)
							.istLeer()) {
						return false;
					}
				}
			}
		} else if (vonFigur.istKreisGross()) {
			if (vonZeile == nachZeile || vonSpalte == nachSpalte) { // wie
				// Quadrat
				if (vonZeile < nachZeile) {
					for (int i = vonZeile + 1; i < nachZeile; i++) {
						if (!this.brett.liefereFeld(i, vonSpalte).istLeer()) {
							return false;
						}
					}
				} else if (vonZeile > nachZeile) {
					for (int i = vonZeile - 1; i > nachZeile; i--) {
						if (!this.brett.liefereFeld(i, vonSpalte).istLeer()) {
							return false;
						}
					}
				} else if (vonSpalte < nachSpalte) {
					for (int i = vonSpalte + 1; i < nachSpalte; i++) {
						if (!this.brett.liefereFeld(vonZeile, i).istLeer()) {
							return false;
						}
					}
				} else if (vonSpalte > nachSpalte) {
					for (int i = vonSpalte - 1; i > nachSpalte; i--) {
						if (!this.brett.liefereFeld(vonZeile, i).istLeer()) {
							return false;
						}
					}
				}
			} else { // wie Dreieck
				if (vonZeile < nachZeile && vonSpalte < nachSpalte) {
					for (int i = 1; i < nachZeile - vonZeile; i++) {
						if (!this.brett
								.liefereFeld(vonZeile + i, vonSpalte + i)
								.istLeer()) {
							return false;
						}
					}
				} else if (vonZeile > nachZeile && vonSpalte < nachSpalte) {
					for (int i = 1; i < vonZeile - nachZeile; i++) {
						if (!this.brett
								.liefereFeld(vonZeile - i, vonSpalte + i)
								.istLeer()) {
							return false;
						}
					}
				} else if (vonZeile < nachZeile && vonSpalte > nachSpalte) {
					for (int i = 1; i < nachZeile - vonZeile; i++) {
						if (!this.brett
								.liefereFeld(vonZeile + i, vonSpalte - i)
								.istLeer()) {
							return false;
						}
					}
				} else { // beide >
					for (int i = 1; i < vonZeile - nachZeile; i++) {
						if (!this.brett
								.liefereFeld(vonZeile - i, vonSpalte - i)
								.istLeer()) {
							return false;
						}
					}
				}
			}
		}

		// alles ok!
		return true;
	}

	// Ueberpruefung des Spielendes;
	// Uebergeben wird der letzte Spieler und der letzte Spielzug
	// Ergebnis: KEIN_ENDE = 0, SIEGER_WEISS = 1, SIEGER_SCHWARZ = 2
	int spielEnde(OpalaSpieler aktSpieler, OpalaSpielzug aktZug) {

		// hat Spieler die gegnerische Grundlinie erreicht und wurde
		// nicht geschlagen? (hier UEBERPRUEFEN!)
		if (this.merkFigur != null && !this.merkFigur.istGeschlagen()) {
			this.merkFigur = null;
			if (aktSpieler == this.weiss) {
				return OpalaRegeln.SIEGER_SCHWARZ;
			} else {
				return OpalaRegeln.SIEGER_WEISS;
			}
		}

		// besitzt Gegner noch Figuren?
		if (aktSpieler == this.weiss
				&& this.brett.anzahlFiguren(this.schwarz) == 0) {
			return OpalaRegeln.SIEGER_WEISS;
		} else if (aktSpieler == this.schwarz
				&& this.brett.anzahlFiguren(this.weiss) == 0) {
			return OpalaRegeln.SIEGER_SCHWARZ;
		}

		// kann Gegner noch legalen Spielzug ausfuehren?
		// sehr ineffiziente Bestimmung!
		if (aktSpieler == this.weiss) {
			OpalaFigur[] figuren = this.brett.liefereFiguren(this.schwarz);
			boolean zugMoeglich = false;
			legalTest: for (int f = 0; f < figuren.length; f++) {
				for (int z = 0; z < OpalaBrett.GROESSE; z++) {
					for (int s = 0; s < OpalaBrett.GROESSE; s++) {
						OpalaSpielzug probeZug = new OpalaSpielzug(
								figuren[f].liefereZeile(),
								figuren[f].liefereSpalte(), z, s);
						if (this.spielzugOK(probeZug, this.schwarz)) {
							zugMoeglich = true;
							break legalTest;
						}
					}
				}
			}
			if (!zugMoeglich) {
				return OpalaRegeln.SIEGER_WEISS;
			}
		} else {
			OpalaFigur[] figuren = this.brett.liefereFiguren(this.weiss);
			boolean zugMoeglich = false;
			legalTest: for (int f = 0; f < figuren.length; f++) {
				for (int z = 0; z < OpalaBrett.GROESSE; z++) {
					for (int s = 0; s < OpalaBrett.GROESSE; s++) {
						OpalaSpielzug probeZug = new OpalaSpielzug(
								figuren[f].liefereZeile(),
								figuren[f].liefereSpalte(), z, s);
						if (this.spielzugOK(probeZug, this.weiss)) {
							zugMoeglich = true;
							break legalTest;
						}
					}
				}
			}
			if (!zugMoeglich) {
				return OpalaRegeln.SIEGER_SCHWARZ;
			}
		}

		// erreicht Spieler die gegnerische Grundlinie und kann
		// nicht mehr geschlagen werden?
		if (aktSpieler == this.weiss) {
			if (aktZug.nachWohin().liefereZeile() == 0) {
				boolean schlagenMoeglich = false;
				OpalaFigur[] figuren = this.brett.liefereFiguren(this.schwarz);
				for (int f = 0; f < figuren.length; f++) {
					OpalaSpielzug probeZug = new OpalaSpielzug(
							figuren[f].liefereZeile(),
							figuren[f].liefereSpalte(), aktZug.nachWohin()
									.liefereZeile(), aktZug.nachWohin()
									.liefereSpalte());
					if (this.spielzugOK(probeZug, this.schwarz)) {
						schlagenMoeglich = true;
						break;
					}
				}
				if (!schlagenMoeglich) {
					return OpalaRegeln.SIEGER_WEISS;
				}
			}
		} else {
			if (aktZug.nachWohin().liefereZeile() == OpalaBrett.GROESSE - 1) {
				boolean schlagenMoeglich = false;
				OpalaFigur[] figuren = this.brett.liefereFiguren(this.weiss);
				for (int f = 0; f < figuren.length; f++) {
					OpalaSpielzug probeZug = new OpalaSpielzug(
							figuren[f].liefereZeile(),
							figuren[f].liefereSpalte(), aktZug.nachWohin()
									.liefereZeile(), aktZug.nachWohin()
									.liefereSpalte());
					if (this.spielzugOK(probeZug, this.weiss)) {
						schlagenMoeglich = true;
						break;
					}
				}
				if (!schlagenMoeglich) {
					return OpalaRegeln.SIEGER_SCHWARZ;
				}
			}
		}

		// hat Spieler die gegnerische Grundlinie erreicht und wurde
		// nicht geschlagen? (hier MERKEN!)
		if (aktSpieler == this.weiss && aktZug.nachWohin().liefereZeile() == 0) {
			this.merkFigur = this.brett.liefereFeld(
					aktZug.nachWohin().liefereZeile(),
					aktZug.nachWohin().liefereSpalte()).liefereFigur();
		} else if (aktSpieler == this.schwarz
				&& aktZug.nachWohin().liefereZeile() == OpalaBrett.GROESSE - 1) {
			this.merkFigur = this.brett.liefereFeld(
					aktZug.nachWohin().liefereZeile(),
					aktZug.nachWohin().liefereSpalte()).liefereFigur();
		} else {
			this.merkFigur = null;
		}

		// kein Ende
		return OpalaRegeln.KEIN_ENDE;
	}

	// berechnet den Betrag einer Zahl
	static int abs(int zahl) {
		return zahl >= 0 ? zahl : -zahl;
	}

}

// -----------------------------------------------------
// Klasse OpalaSpieler
// -----------------------------------------------------

class OpalaSpieler {

	// Attribute
	boolean istWeiss;

	// Methoden
	// Konstruktor (Initialisierung eines Spielers)
	OpalaSpieler(boolean istWeiss) {
		this.istWeiss = istWeiss;
	}

	// Ueberpruefung, ob es Spieler A ist
	boolean istWeiss() {
		return this.istWeiss;
	}

	// erfragt und liefert naechsten Spielzug des
	// Spielers; uebergeben wird der letzte Zug des
	// Gegners
	OpalaSpielzug liefereNaechstenSpielzug(OpalaSpielzug gegnerZug) {
		return this.erfrageNaechstenSpielzugBeimSpieler();
	}

	// erfragt naechsten Spielzug beim Spieler
	OpalaSpielzug erfrageNaechstenSpielzugBeimSpieler() {
		char vonZeile, vonSpalte, nachZeile, nachSpalte;

		vonZeile = IO.readChar("von Zeile (1-6): ");
		while (vonZeile < '1' || vonZeile > '6') {
			IO.println("Ungueltig!");
			vonZeile = IO.readChar("von Zeile (1-6): ");
		}

		vonSpalte = IO.readChar("von Spalte (A-F): ");
		while (vonSpalte < 'A' || vonSpalte > 'F') {
			IO.println("Ungueltig!");
			vonSpalte = IO.readChar("von Spalte (A-F): ");
		}

		nachZeile = IO.readChar("nach Zeile (1-6): ");
		while (nachZeile < '1' || nachZeile > '6') {
			IO.println("Ungueltig!");
			nachZeile = IO.readChar("nach Zeile (1-6): ");
		}

		nachSpalte = IO.readChar("nach Spalte (A-F): ");
		while (nachSpalte < 'A' || nachSpalte > 'F') {
			IO.println("Ungueltig!");
			nachSpalte = IO.readChar("nach Spalte (A-F): ");
		}

		return new OpalaSpielzug(vonZeile, vonSpalte, nachZeile, nachSpalte);
	}

}
