import java.util.ArrayList;

public class UE11Aufgabe18 {

	public static void main(String[] args) {
		SmessSpieler spielerA = new SmessSpieler(true);
		SmessSpieler spielerB = new SmessSpieler(false);
		SmessBrett brett = new SmessBrett();
		SmessRegeln regeln = new SmessRegeln(brett);

		Smess smess = new Smess(brett, regeln, spielerA, spielerB);
		smess.spielen();
	}

}

class Smess {

	SmessBrett brett;

	SmessRegeln regeln;

	SmessSpieler[] spieler;

	Smess(SmessBrett brett, SmessRegeln regeln, SmessSpieler spielerA,
			SmessSpieler spielerB) {
		this.brett = brett;
		this.regeln = regeln;
		this.spieler = new SmessSpieler[2];
		this.spieler[0] = spielerA;
		this.spieler[1] = spielerB;
	}

	void spielen() {
		this.brett.print();
		int aktSpielerIndex = 0;
		SmessSpielzug zug = null;
		while (!this.regeln.spielEnde(this.spieler[aktSpielerIndex])) {
			System.out.println("Spieler " + this.spieler[aktSpielerIndex]
					+ " ist am Zug!");
			// this.regeln.printZuege(this.spieler[aktSpielerIndex]);
			zug = this.spieler[aktSpielerIndex].naechsterSpielzug(zug);
			if (!this.regeln.spielzugOk(zug, this.spieler[aktSpielerIndex])) {
				System.out.println("Ungltiger Spielzug (" + zug + ")!");
				System.out.println("Spieler " + this.spieler[aktSpielerIndex]
						+ " hat verloren!");
				return;
			}
			this.brett.fuehreSpielzugAus(zug);
			this.brett.print();
			aktSpielerIndex = (aktSpielerIndex + 1) % 2;
		}

		System.out.println("Spielende!");

		int grund = this.regeln.getEndeGrund();
		switch (grund) {
		case SmessRegeln.BRAIN_GESCHLAGEN:
			System.out.println("Grund: Brain wurde geschlagen!");
			break;
		case SmessRegeln.KEIN_ZUG_MOEGLICH:
			System.out.println("Grund: Kein Zug mehr mglich!");
			break;
		case SmessRegeln.NUR_NOCH_BRAINS:
			System.out.println("Grund: Nur noch zwei Brains auf dem Feld!");
			break;
		}

		int ergebnis = this.regeln.getErgebnis();
		switch (ergebnis) {
		case SmessRegeln.SIEGER_A:
			System.out.println("Spieler A hat gewonnen!");
			break;
		case SmessRegeln.SIEGER_B:
			System.out.println("Spieler B hat gewonnen!");
			break;
		case SmessRegeln.UNENTSCHIEDEN:
			System.out.println("Unentschieden!");
		}

	}

}

class SmessRegeln {

	static final int SIEGER_A = -1;

	static final int UNENTSCHIEDEN = 0;

	static final int SIEGER_B = 1;

	static final int KEIN_ZUG_MOEGLICH = 0;

	static final int BRAIN_GESCHLAGEN = 1;

	static final int NUR_NOCH_BRAINS = 2;

	SmessBrett brett;

	int sieger;

	int grund;

	SmessRegeln(SmessBrett brett) {
		this.brett = brett;
		this.grund = -2;
		this.sieger = -2;
	}

	boolean spielzugOk(SmessSpielzug zug, SmessSpieler spieler) {

		// Koordinaten ok
		if (zug.getVonReihe() < 1 || zug.getVonReihe() > 8) {
			return false;
		}
		if (zug.getVonSpalte() < 'a' || zug.getVonSpalte() > 'g') {
			return false;
		}
		if (zug.getNachReihe() < 1 || zug.getNachReihe() > 8) {
			return false;
		}
		if (zug.getNachSpalte() < 'a' || zug.getNachSpalte() > 'g') {
			return false;
		}

		SmessFeld vonFeld = this.brett.getFeld(zug.getVonReihe(),
				zug.getVonSpalte());
		SmessFeld nachFeld = this.brett.getFeld(zug.getNachReihe(),
				zug.getNachSpalte());

		// keine Figur auf angegebenem Feld
		if (!vonFeld.hatFigur()) {
			return false;
		}
		SmessFigur figur = vonFeld.getFigur();

		// Figur des Gegners
		if (figur.isA() != spieler.isSpielerA()) {
			return false;
		}

		// gueltiges Feld
		SmessFeld[] nachFelder = this.getZugFelder(vonFeld, spieler);
		for (int f = 0; f < nachFelder.length; f++) {
			if (nachFelder[f] == nachFeld) { // == geht hier
				return true;
			}
		}

		return false;
	}

	SmessFeld[] getZugFelder(SmessFeld vonFeld, SmessSpieler aktSpieler) {
		// Feld leer
		if (!vonFeld.hatFigur()) {
			return new SmessFeld[0];
		}

		// Feld mit fremder Figur
		SmessFigur figur = vonFeld.getFigur();
		if (figur.isA() != aktSpieler.isSpielerA()) {
			return new SmessFeld[0];
		}

		if (figur.getClass() == Brain.class || figur.getClass() == Ninny.class) {
			return this.bearbeiteNinnyOderBrainZug(vonFeld, figur);
		} else {
			return this.bearbeiteNumskullZug(vonFeld, figur);
		}

	}

	SmessFeld[] bearbeiteNinnyOderBrainZug(SmessFeld vonFeld, SmessFigur figur) {
		ArrayList<SmessFeld> nachFelder = new ArrayList<SmessFeld>();

		int[] richtungen = vonFeld.getRichtungen();
		SmessFeld nachFeld = null;
		for (int r = 0; r < richtungen.length; r++) {
			switch (richtungen[r]) {
			case SmessRichtung.NORD:
				nachFeld = this.brett.getFeld(vonFeld.getReihe() + 1,
						vonFeld.getSpalte());
				break;
			case SmessRichtung.NORDWEST:
				nachFeld = this.brett.getFeld(vonFeld.getReihe() + 1,
						(char) (vonFeld.getSpalte() - 1));
				break;
			case SmessRichtung.WEST:
				nachFeld = this.brett.getFeld(vonFeld.getReihe(),
						(char) (vonFeld.getSpalte() - 1));
				break;
			case SmessRichtung.SUEDWEST:
				nachFeld = this.brett.getFeld(vonFeld.getReihe() - 1,
						(char) (vonFeld.getSpalte() - 1));
				break;
			case SmessRichtung.SUED:
				nachFeld = this.brett.getFeld(vonFeld.getReihe() - 1,
						vonFeld.getSpalte());
				break;
			case SmessRichtung.SUEDOST:
				nachFeld = this.brett.getFeld(vonFeld.getReihe() - 1,
						(char) (vonFeld.getSpalte() + 1));
				break;
			case SmessRichtung.OST:
				nachFeld = this.brett.getFeld(vonFeld.getReihe(),
						(char) (vonFeld.getSpalte() + 1));
				break;
			case SmessRichtung.NORDOST:
				nachFeld = this.brett.getFeld(vonFeld.getReihe() + 1,
						(char) (vonFeld.getSpalte() + 1));
				break;
			}
			if (!nachFeld.hatFigur() || nachFeld.hatFigur()
					&& nachFeld.getFigur().isA() != figur.isA()) {
				nachFelder.add(nachFeld);
			}
		}

		return nachFelder.toArray(new SmessFeld[0]);
	}

	SmessFeld[] bearbeiteNumskullZug(SmessFeld vonFeld, SmessFigur figur) {
		ArrayList<SmessFeld> nachFelder = new ArrayList<SmessFeld>();

		int[] richtungen = vonFeld.getRichtungen();
		SmessFeld nachFeld = null;
		richtungsSchleife: for (int r = 0; r < richtungen.length; r++) {
			switch (richtungen[r]) {
			case SmessRichtung.NORD:
				for (int reihe = vonFeld.getReihe() + 1; reihe <= 8; reihe++) {
					nachFeld = this.brett.getFeld(reihe, vonFeld.getSpalte());
					if (!nachFeld.hatFigur()) {
						nachFelder.add(nachFeld);
					} else if (nachFeld.getFigur().isA() != figur.isA()) {
						nachFelder.add(nachFeld);
						continue richtungsSchleife;
					} else {
						continue richtungsSchleife;
					}
				}
				break;
			case SmessRichtung.NORDWEST:
				for (int reihe = vonFeld.getReihe() + 1, spalte = vonFeld
						.getSpalte() - 1; reihe <= 8 && (char) spalte >= 'a'; reihe++, spalte--) {
					nachFeld = this.brett.getFeld(reihe, (char) spalte);
					if (!nachFeld.hatFigur()) {
						nachFelder.add(nachFeld);
					} else if (nachFeld.getFigur().isA() != figur.isA()) {
						nachFelder.add(nachFeld);
						continue richtungsSchleife;
					} else {
						continue richtungsSchleife;
					}
				}
				break;
			case SmessRichtung.WEST:
				for (int spalte = vonFeld.getSpalte() - 1; (char) spalte >= 'a'; spalte--) {
					nachFeld = this.brett.getFeld(vonFeld.getReihe(),
							(char) spalte);
					if (!nachFeld.hatFigur()) {
						nachFelder.add(nachFeld);
					} else if (nachFeld.getFigur().isA() != figur.isA()) {
						nachFelder.add(nachFeld);
						continue richtungsSchleife;
					} else {
						continue richtungsSchleife;
					}
				}
				break;
			case SmessRichtung.SUEDWEST:
				for (int reihe = vonFeld.getReihe() - 1, spalte = vonFeld
						.getSpalte() - 1; reihe >= 1 && (char) spalte >= 'a'; reihe--, spalte--) {
					nachFeld = this.brett.getFeld(reihe, (char) spalte);
					if (!nachFeld.hatFigur()) {
						nachFelder.add(nachFeld);
					} else if (nachFeld.getFigur().isA() != figur.isA()) {
						nachFelder.add(nachFeld);
						continue richtungsSchleife;
					} else {
						continue richtungsSchleife;
					}
				}
				break;
			case SmessRichtung.SUED:
				for (int reihe = vonFeld.getReihe() - 1; reihe >= 1; reihe--) {
					nachFeld = this.brett.getFeld(reihe, vonFeld.getSpalte());
					if (!nachFeld.hatFigur()) {
						nachFelder.add(nachFeld);
					} else if (nachFeld.getFigur().isA() != figur.isA()) {
						nachFelder.add(nachFeld);
						continue richtungsSchleife;
					} else {
						continue richtungsSchleife;
					}
				}
				break;
			case SmessRichtung.SUEDOST:
				for (int reihe = vonFeld.getReihe() - 1, spalte = vonFeld
						.getSpalte() + 1; reihe >= 1 && (char) spalte <= 'g'; reihe--, spalte++) {
					nachFeld = this.brett.getFeld(reihe, (char) spalte);
					if (!nachFeld.hatFigur()) {
						nachFelder.add(nachFeld);
					} else if (nachFeld.getFigur().isA() != figur.isA()) {
						nachFelder.add(nachFeld);
						continue richtungsSchleife;
					} else {
						continue richtungsSchleife;
					}
				}
				break;
			case SmessRichtung.OST:
				for (int spalte = vonFeld.getSpalte() + 1; (char) spalte <= 'g'; spalte++) {
					nachFeld = this.brett.getFeld(vonFeld.getReihe(),
							(char) spalte);
					if (!nachFeld.hatFigur()) {
						nachFelder.add(nachFeld);
					} else if (nachFeld.getFigur().isA() != figur.isA()) {
						nachFelder.add(nachFeld);
						continue richtungsSchleife;
					} else {
						continue richtungsSchleife;
					}
				}
				break;
			case SmessRichtung.NORDOST:
				for (int reihe = vonFeld.getReihe() + 1, spalte = vonFeld
						.getSpalte() + 1; reihe <= 8 && (char) spalte <= 'g'; reihe++, spalte++) {
					nachFeld = this.brett.getFeld(reihe, (char) spalte);
					if (!nachFeld.hatFigur()) {
						nachFelder.add(nachFeld);
					} else if (nachFeld.getFigur().isA() != figur.isA()) {
						nachFelder.add(nachFeld);
						continue richtungsSchleife;
					} else {
						continue richtungsSchleife;
					}
				}
				break;
			}
		}

		return nachFelder.toArray(new SmessFeld[0]);
	}

	boolean spielEnde(SmessSpieler aktSpieler) {

		// Brain geschlagen
		int anzahlBrains = 0;
		for (int r = 1; r <= 8; r++) {
			for (char s = 'a'; s <= 'g'; s++) {
				SmessFeld feld = this.brett.getFeld(r, s);
				if (feld.hatFigur()
						&& feld.getFigur().getClass() == Brain.class) {
					anzahlBrains++;
				}
			}
		}
		if (anzahlBrains < 2) {
			if (aktSpieler.isSpielerA()) {
				this.sieger = SmessRegeln.SIEGER_B;
			} else {
				this.sieger = SmessRegeln.SIEGER_A;
			}
			this.grund = SmessRegeln.BRAIN_GESCHLAGEN;
			return true;
		}

		// nur noch 2 Brains
		int anzahlFiguren = 0;
		for (int r = 1; r <= 8; r++) {
			for (char s = 'a'; s <= 'g'; s++) {
				SmessFeld feld = this.brett.getFeld(r, s);
				if (feld.hatFigur()) {
					anzahlFiguren++;
				}
			}
		}
		if (anzahlFiguren <= 2) {
			this.sieger = SmessRegeln.UNENTSCHIEDEN;
			this.grund = SmessRegeln.NUR_NOCH_BRAINS;
			return true;
		}

		// kein Zug mglich
		for (int r = 1; r <= 8; r++) {
			for (char s = 'a'; s <= 'g'; s++) {
				SmessFeld feld = this.brett.getFeld(r, s);
				SmessFeld[] nachFelder = this.getZugFelder(feld, aktSpieler);
				if (nachFelder != null && nachFelder.length > 0) {
					return false;
				}
			}
		}
		if (aktSpieler.isSpielerA()) {
			this.sieger = SmessRegeln.SIEGER_B;
		} else {
			this.sieger = SmessRegeln.SIEGER_A;
		}
		this.grund = SmessRegeln.KEIN_ZUG_MOEGLICH;
		return true;
	}

	int getErgebnis() {
		return this.sieger;
	}

	int getEndeGrund() {
		return this.grund;
	}

	void printZuege(SmessSpieler aktSpieler) {
		for (int r = 1; r <= 8; r++) {
			for (char s = 'a'; s <= 'g'; s++) {
				SmessFeld feld = this.brett.getFeld(r, s);
				SmessFeld[] nachFelder = this.getZugFelder(feld, aktSpieler);
				if (nachFelder != null && nachFelder.length > 0) {
					System.out.print("R=" + feld.getReihe() + ",S="
							+ feld.getSpalte() + ": ");
					for (int f = 0; f < nachFelder.length; f++) {
						System.out.print("(" + nachFelder[f].getReihe() + ","
								+ nachFelder[f].getSpalte() + ") ");
					}
					System.out.println();
				}
			}
		}
	}

}

class SmessRichtung {

	static final int NORD = 1;

	static final int NORDWEST = 2;

	static final int WEST = 3;

	static final int SUEDWEST = 4;

	static final int SUED = 5;

	static final int SUEDOST = 6;

	static final int OST = 7;

	static final int NORDOST = 8;

}

class SmessBrett {

	SmessFeld[][] brett;

	SmessBrett() {
		this.brett = new SmessFeld[8][7];

		// erste Reihe
		this.brett[0][0] = new SmessFeld(8, 'a', this, SmessRichtung.SUED,
				SmessRichtung.OST);
		this.brett[0][1] = new SmessFeld(8, 'b', this, SmessRichtung.WEST,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[0][2] = new SmessFeld(8, 'c', this, SmessRichtung.WEST,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[0][3] = new SmessFeld(8, 'd', this, SmessRichtung.WEST,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[0][4] = new SmessFeld(8, 'e', this, SmessRichtung.WEST,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[0][5] = new SmessFeld(8, 'f', this, SmessRichtung.WEST,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[0][6] = new SmessFeld(8, 'g', this, SmessRichtung.SUED);

		// zweite Reihe
		this.brett[1][0] = new SmessFeld(7, 'a', this, SmessRichtung.NORD,
				SmessRichtung.SUED);
		this.brett[1][1] = new SmessFeld(7, 'b', this, SmessRichtung.NORDWEST,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[1][2] = new SmessFeld(7, 'c', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[1][3] = new SmessFeld(7, 'd', this, SmessRichtung.WEST,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[1][4] = new SmessFeld(7, 'e', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[1][5] = new SmessFeld(7, 'f', this, SmessRichtung.WEST,
				SmessRichtung.SUED);
		this.brett[1][6] = new SmessFeld(7, 'g', this, SmessRichtung.NORD,
				SmessRichtung.SUED);

		// dritte Reihe
		this.brett[2][0] = new SmessFeld(6, 'a', this, SmessRichtung.NORD,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[2][1] = new SmessFeld(6, 'b', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[2][2] = new SmessFeld(6, 'c', this, SmessRichtung.NORDWEST,
				SmessRichtung.SUEDWEST, SmessRichtung.SUEDOST,
				SmessRichtung.NORDOST);
		this.brett[2][3] = new SmessFeld(6, 'd', this, SmessRichtung.WEST,
				SmessRichtung.OST);
		this.brett[2][4] = new SmessFeld(6, 'e', this, SmessRichtung.NORDWEST,
				SmessRichtung.SUEDWEST, SmessRichtung.SUEDOST,
				SmessRichtung.NORDOST);
		this.brett[2][5] = new SmessFeld(6, 'f', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.OST);
		this.brett[2][6] = new SmessFeld(6, 'g', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED);

		// vierte Reihe
		this.brett[3][0] = new SmessFeld(5, 'a', this, SmessRichtung.SUEDOST,
				SmessRichtung.OST, SmessRichtung.NORDOST);
		this.brett[3][1] = new SmessFeld(5, 'b', this, SmessRichtung.OST);
		this.brett[3][2] = new SmessFeld(5, 'c', this, SmessRichtung.NORD,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[3][3] = new SmessFeld(5, 'd', this, SmessRichtung.NORD,
				SmessRichtung.NORDWEST, SmessRichtung.WEST,
				SmessRichtung.SUEDWEST, SmessRichtung.SUED,
				SmessRichtung.SUEDOST, SmessRichtung.OST, SmessRichtung.NORDOST);
		this.brett[3][4] = new SmessFeld(5, 'e', this, SmessRichtung.NORD,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[3][5] = new SmessFeld(5, 'f', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[3][6] = new SmessFeld(5, 'g', this, SmessRichtung.NORD,
				SmessRichtung.SUED);

		// fuenfte Reihe
		this.brett[4][0] = new SmessFeld(4, 'a', this, SmessRichtung.NORD,
				SmessRichtung.SUED);
		this.brett[4][1] = new SmessFeld(4, 'b', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[4][2] = new SmessFeld(4, 'c', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED);
		this.brett[4][3] = new SmessFeld(4, 'd', this, SmessRichtung.NORD,
				SmessRichtung.NORDWEST, SmessRichtung.WEST,
				SmessRichtung.SUEDWEST, SmessRichtung.SUED,
				SmessRichtung.SUEDOST, SmessRichtung.OST, SmessRichtung.NORDOST);
		this.brett[4][4] = new SmessFeld(4, 'e', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED);
		this.brett[4][5] = new SmessFeld(4, 'f', this, SmessRichtung.WEST);
		this.brett[4][6] = new SmessFeld(4, 'g', this, SmessRichtung.NORDWEST,
				SmessRichtung.WEST, SmessRichtung.SUEDWEST);

		// sechste Reihe
		this.brett[5][0] = new SmessFeld(3, 'a', this, SmessRichtung.NORD,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[5][1] = new SmessFeld(3, 'b', this, SmessRichtung.WEST,
				SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[5][2] = new SmessFeld(3, 'c', this, SmessRichtung.NORDWEST,
				SmessRichtung.SUEDWEST, SmessRichtung.SUEDOST,
				SmessRichtung.NORDOST);
		this.brett[5][3] = new SmessFeld(3, 'd', this, SmessRichtung.WEST,
				SmessRichtung.OST);
		this.brett[5][4] = new SmessFeld(3, 'e', this, SmessRichtung.NORDWEST,
				SmessRichtung.SUEDWEST, SmessRichtung.SUEDOST,
				SmessRichtung.NORDOST);
		this.brett[5][5] = new SmessFeld(3, 'f', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[5][6] = new SmessFeld(3, 'g', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED);

		// siebte Reihe
		this.brett[6][0] = new SmessFeld(2, 'a', this, SmessRichtung.NORD,
				SmessRichtung.SUED);
		this.brett[6][1] = new SmessFeld(2, 'b', this, SmessRichtung.NORD,
				SmessRichtung.OST);
		this.brett[6][2] = new SmessFeld(2, 'c', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[6][3] = new SmessFeld(2, 'd', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.OST);
		this.brett[6][4] = new SmessFeld(2, 'e', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUED, SmessRichtung.OST);
		this.brett[6][5] = new SmessFeld(2, 'f', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.SUEDOST);
		this.brett[6][6] = new SmessFeld(2, 'g', this, SmessRichtung.NORD,
				SmessRichtung.SUED);

		// achte Reihe
		this.brett[7][0] = new SmessFeld(1, 'a', this, SmessRichtung.NORD);
		this.brett[7][1] = new SmessFeld(1, 'b', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.OST);
		this.brett[7][2] = new SmessFeld(1, 'c', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.OST);
		this.brett[7][3] = new SmessFeld(1, 'd', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.OST);
		this.brett[7][4] = new SmessFeld(1, 'e', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.OST);
		this.brett[7][5] = new SmessFeld(1, 'f', this, SmessRichtung.NORD,
				SmessRichtung.WEST, SmessRichtung.OST);
		this.brett[7][6] = new SmessFeld(1, 'g', this, SmessRichtung.NORD,
				SmessRichtung.WEST);

		// besetzen
		this.brett[0][1].setFigur(new Numskull(false));
		this.brett[0][2].setFigur(new Numskull(false));
		this.brett[0][3].setFigur(new Brain(false));
		this.brett[0][4].setFigur(new Numskull(false));
		this.brett[0][5].setFigur(new Numskull(false));
		this.brett[1][0].setFigur(new Ninny(false));
		this.brett[1][1].setFigur(new Ninny(false));
		this.brett[1][2].setFigur(new Ninny(false));
		this.brett[1][3].setFigur(new Ninny(false));
		this.brett[1][4].setFigur(new Ninny(false));
		this.brett[1][5].setFigur(new Ninny(false));
		this.brett[1][6].setFigur(new Ninny(false));

		this.brett[6][0].setFigur(new Ninny(true));
		this.brett[6][1].setFigur(new Ninny(true));
		this.brett[6][2].setFigur(new Ninny(true));
		this.brett[6][3].setFigur(new Ninny(true));
		this.brett[6][4].setFigur(new Ninny(true));
		this.brett[6][5].setFigur(new Ninny(true));
		this.brett[6][6].setFigur(new Ninny(true));
		this.brett[7][1].setFigur(new Numskull(true));
		this.brett[7][2].setFigur(new Numskull(true));
		this.brett[7][3].setFigur(new Brain(true));
		this.brett[7][4].setFigur(new Numskull(true));
		this.brett[7][5].setFigur(new Numskull(true));
	}

	SmessFeld getFeld(int reihe, char spalte) {
		if (reihe < 1 || reihe > 8 || spalte < 'a' || spalte > 'g') {
			return null;
		}
		return this.brett[8 - reihe][spalte - 'a'];
	}

	void fuehreSpielzugAus(SmessSpielzug zug) {
		SmessFeld vonFeld = this.brett[8 - zug.getVonReihe()][zug
				.getVonSpalte() - 'a'];
		SmessFeld nachFeld = this.brett[8 - zug.getNachReihe()][zug
				.getNachSpalte() - 'a'];
		SmessFigur figur = vonFeld.getFigur();
		vonFeld.entferneFigur();
		if (nachFeld.hatFigur()) {
			nachFeld.entferneFigur();
		}
		if (this.checkNinnyUmwandlung(figur, nachFeld)) {
			figur = new Numskull(figur.isA());
		}
		nachFeld.setFigur(figur);
	}

	void print() {
		System.out.println("   Spieler B (b, n, y)");
		System.out.print("  ");
		for (int s = 0; s < this.brett[0].length; s++) {
			System.out.print(" ");
			System.out.print(this.brett[0][s].getSpalte());
			System.out.print("  ");
		}
		System.out.println();

		System.out.print(" +");
		for (int s = 0; s < this.brett[0].length; s++) {
			System.out.print("---+");
		}
		System.out.println();

		for (int r = 0; r < this.brett.length; r++) {
			this.printReihe(r);
		}
		System.out.println("   Spieler A (B, N, Y)");
		System.out.println();
	}

	void printReihe(int reihe) {

		// erste Zeile
		System.out.print(" |");
		for (int s = 0; s < this.brett[reihe].length; s++) {
			if (this.brett[reihe][s].besitztRichtung(SmessRichtung.NORDWEST)) {
				System.out.print("\\");
			} else {
				System.out.print(" ");
			}
			if (this.brett[reihe][s].besitztRichtung(SmessRichtung.NORD)) {
				System.out.print("|");
			} else {
				System.out.print(" ");
			}
			if (this.brett[reihe][s].besitztRichtung(SmessRichtung.NORDOST)) {
				System.out.print("/");
			} else {
				System.out.print(" ");
			}
			System.out.print("|");
		}
		System.out.println();

		// zweite Zeile
		System.out.print(this.brett[reihe][0].getReihe());
		System.out.print("|");
		for (int s = 0; s < this.brett[reihe].length; s++) {
			if (this.brett[reihe][s].besitztRichtung(SmessRichtung.WEST)) {
				System.out.print("-");
			} else {
				System.out.print(" ");
			}
			if (this.brett[reihe][s].hatFigur()) {
				System.out.print(this.brett[reihe][s].getFigur());
			} else {
				System.out.print(" ");
			}

			if (this.brett[reihe][s].besitztRichtung(SmessRichtung.OST)) {
				System.out.print("-");
			} else {
				System.out.print(" ");
			}
			System.out.print("|");
		}
		System.out.println();

		// dritte Zeile
		System.out.print(" |");
		for (int s = 0; s < this.brett[reihe].length; s++) {
			if (this.brett[reihe][s].besitztRichtung(SmessRichtung.SUEDWEST)) {
				System.out.print("/");
			} else {
				System.out.print(" ");
			}
			if (this.brett[reihe][s].besitztRichtung(SmessRichtung.SUED)) {
				System.out.print("|");
			} else {
				System.out.print(" ");
			}
			if (this.brett[reihe][s].besitztRichtung(SmessRichtung.SUEDOST)) {
				System.out.print("\\");
			} else {
				System.out.print(" ");
			}
			System.out.print("|");
		}
		System.out.println();

		// vierte Zeile
		System.out.print(" +");
		for (int s = 0; s < this.brett[reihe].length; s++) {
			System.out.print("---+");
		}
		System.out.println();
	}

	boolean checkNinnyUmwandlung(SmessFigur figur, SmessFeld feld) {
		if (!(figur.getClass() == Ninny.class)) {
			return false;
		}
		boolean figurA = figur.isA();
		int reihe = feld.getReihe();
		char spalte = feld.getSpalte();
		if (figurA) {
			return reihe == 8
					&& (spalte == 'b' || spalte == 'c' || spalte == 'e' || spalte == 'f');
		} else {
			return reihe == 1
					&& (spalte == 'b' || spalte == 'c' || spalte == 'e' || spalte == 'f');
		}
	}

}

class SmessFeld {

	int reihe;

	char spalte;

	int[] richtungen; // speichert die Richtungspfeile des Feldes

	SmessFigur figur;

	SmessBrett brett;

	SmessFeld(int reihe, char spalte, SmessBrett brett, int... richtungen) {
		this.reihe = reihe;
		this.spalte = spalte;
		this.brett = brett;
		this.richtungen = new int[richtungen.length];
		for (int r = 0; r < richtungen.length; r++) {
			this.richtungen[r] = richtungen[r];
		}
		this.figur = null;
	}

	int getReihe() {
		return this.reihe;
	}

	char getSpalte() {
		return this.spalte;
	}

	boolean besitztRichtung(int richtung) {
		for (int r : this.richtungen) {
			if (r == richtung) {
				return true;
			}
		}
		return false;
	}

	int[] getRichtungen() {
		return this.richtungen;
	}

	SmessFigur getFigur() {
		return this.figur;
	}

	void setFigur(SmessFigur figur) {
		this.figur = figur;
	}

	void entferneFigur() {
		this.figur = null;
	}

	boolean hatFigur() {
		return this.figur != null;
	}

	SmessBrett getBrett() {
		return this.brett;
	}

}

class SmessSpieler {

	boolean spielerA;

	SmessSpieler(boolean spielerA) {
		this.spielerA = spielerA;
	}

	boolean isSpielerA() {
		return this.spielerA;
	}

	boolean isSpielerB() {
		return !this.isSpielerA();
	}

	@Override
	public String toString() {
		return this.spielerA ? "A" : "B";
	}

	SmessSpielzug naechsterSpielzug(SmessSpielzug gegnerZug) {
		int vonReihe = IO.readInt("Von Reihe (1..8): ");
		char vonSpalte = IO.readChar("Von Spalte (a..g):");
		int nachReihe = IO.readInt("Nach Reihe (1..8): ");
		char nachSpalte = IO.readChar("Nach Spalte (a..g):");
		return new SmessSpielzug(vonReihe, vonSpalte, nachReihe, nachSpalte);
	}

}

class SmessSpielzug {

	int vonReihe;

	char vonSpalte;

	int nachReihe;

	char nachSpalte;

	SmessSpielzug(int vonReihe, char vonSpalte, int nachReihe, char nachSpalte) {
		this.vonReihe = vonReihe;
		this.vonSpalte = vonSpalte;
		this.nachReihe = nachReihe;
		this.nachSpalte = nachSpalte;
	}

	int getVonReihe() {
		return this.vonReihe;
	}

	char getVonSpalte() {
		return this.vonSpalte;
	}

	int getNachReihe() {
		return this.nachReihe;
	}

	char getNachSpalte() {
		return this.nachSpalte;
	}

	@Override
	public String toString() {
		return "(" + this.vonReihe + "," + this.vonSpalte + ") --> ("
				+ this.nachReihe + "," + this.nachSpalte + ")";
	}

}

abstract class SmessFigur {

	boolean isA;

	SmessFigur(boolean isA) {
		this.isA = isA;
	}

	boolean isA() {
		return this.isA;
	}

}

class Brain extends SmessFigur {

	static char A_SYMBOL = 'B';

	static char B_SYMBOL = 'b';

	Brain(boolean isA) {
		super(isA);
	}

	@Override
	public String toString() {
		return "" + (this.isA ? Brain.A_SYMBOL : Brain.B_SYMBOL);
	}

}

class Ninny extends SmessFigur {

	static char A_SYMBOL = 'Y';

	static char B_SYMBOL = 'y';

	Ninny(boolean isA) {
		super(isA);
	}

	@Override
	public String toString() {
		return "" + (this.isA ? Ninny.A_SYMBOL : Ninny.B_SYMBOL);
	}

}

class Numskull extends SmessFigur {

	static char A_SYMBOL = 'N';

	static char B_SYMBOL = 'n';

	Numskull(boolean isA) {
		super(isA);
	}

	@Override
	public String toString() {
		return "" + (this.isA ? Numskull.A_SYMBOL : Numskull.B_SYMBOL);
	}

}
